import { createEffect, type Component, createSignal, Show, For } from 'solid-js';

import styles from './App.module.css';
import { A, HashRouter, Route, RouteSectionProps, useNavigate } from '@solidjs/router';
import Login from './pages/Login';
import Groups from './pages/Groups';
import Account from './pages/Account';
import GroupPage from './pages/GroupPage'

import { BsArrowBarRight, BsBookmarkHeartFill, BsPeopleFill } from 'solid-icons/bs';
import { API, api, setAPI, setAuthConfig } from './api';
import { dialogs, localState, setLocalState, showMessageDialog } from './state';
import Dialog from './components/Dialog';
import { errorToString } from './util';

export interface APIConfig {
  apiURL: string,
}

const AppLayout: Component<RouteSectionProps> = (props) => {
  const navigate = useNavigate();

  const [loading, setLoading] = createSignal(true);

  createEffect(async () => {
    try {
      let config: APIConfig = await (await fetch('/config.json')).json();
      setAPI(new API(config.apiURL));

      setAuthConfig(await api().authConfig());

      let search = new URLSearchParams(window.location.search);
      if (search.has("code")) {
        console.log("code");
        let code = search.get("code");
        try {
          let info = await api().login(code!);
          setLocalState({ accountInfo: info })
          window.location.search = "";
        } catch (e) {
          showMessageDialog("Failed to log in", errorToString(e));
        }
      }
    } catch (e) {
      showMessageDialog("Failed to load", errorToString(e));
    }

    setLoading(false);
  }, []);

  const logout = () => {
    navigate("/");
    setLocalState({ accountInfo: null });
  };

  return (
    <div class={styles.App}>
      <For each={dialogs()}>{d => <Dialog {...d} />}</For>
      <Show when={!loading()} fallback={<div>Loading...</div>}>
        <header class={styles.header}>
          <div></div>
          <nav class={styles.navigation}>
            <A href='/wishlists'><BsBookmarkHeartFill class={styles.icon} />Wishlists</A>
            <A href='/groups'><BsPeopleFill class={styles.icon} /> Groups</A>
          </nav>
          <div class={styles.loggedIn}>
            {
              localState.accountInfo != null ?
                <>
                  <A href='/account'>{localState.accountInfo.user.displayName}</A>
                  <button onClick={logout}><BsArrowBarRight class={styles.icon} /></button>
                </> :
                <A href='/login'>Not Logged In</A>
            }
          </div>
        </header>
        <div class={styles.pageContainer}>
          <div class={styles.pageContent}>
            {props.children}
          </div>
        </div>
      </Show>
    </div >
  )
};

const App: Component = () => {
  return (
    <HashRouter root={AppLayout}>
      <Route path={['/', 'wishlists']} component={Wishlists} />
      <Route path='wishlist/:id' component={WishlistPage} />
      <Route path='groups' component={Groups} />
      <Route path='group/:id' component={GroupPage} />
      {localState.accountInfo != null ?
        <>
          <Route path='account' component={Account} />
        </> :
        <>
          <Route path='login' component={Login} />
        </>
      }
    </HashRouter >
  );
};

export default App;
